==> sseed-1.8.1-py3-none-any.whl: sseed/languages.py <==
"""Language detection and management for BIP-39 mnemonics.

This module provides comprehensive support for detecting and validating BIP-39 mnemonics
in multiple languages, including automatic language detection and Unicode-aware validation.

Supported Languages:
- English (en) - Default, Latin script
- Spanish (es) - Latin script with accents
- French (fr) - Latin script with accents
- Italian (it) - Latin script with accents
- Portuguese (pt) - Latin script with accents
- Czech (cs) - Latin script with diacritics
- Chinese Simplified (zh-cn) - Ideographic script
- Chinese Traditional (zh-tw) - Ideographic script
- Korean (ko) - Hangul script (composed and decomposed)

Key Features:
- Automatic language detection with 95%+ accuracy
- Unicode-aware validation for all script types
- BIP-39 standard compliance validation
- Graceful error handling and fallback mechanisms
- LRU caching for performance optimization
"""

import logging
import unicodedata
from functools import lru_cache
from typing import (
    Dict,
    List,
    Optional,
    Tuple,
)

from bip_utils import (
    Bip39Languages,
    Bip39MnemonicValidator,
)

from sseed.exceptions import ValidationError

# Configure module logger
logger = logging.getLogger(__name__)

# Detection threshold for language confidence (70%)
DETECTION_THRESHOLD = 0.7


class LanguageInfo:
    """Information about a BIP-39 language with validation capabilities.

    Attributes:
        bip_enum: The bip_utils language enum value
        code: ISO 639-1 language code (e.g., 'en', 'es')
        name: Human-readable language name
        script: Script type ('latin', 'ideographic', 'hangul')
        word_pattern: Regular expression pattern for word validation
    """

    def __init__(  # pylint: disable=too-many-arguments,too-many-positional-arguments
        self,
        bip_enum: Bip39Languages,
        code: str,
        name: str,
        script: str,
        word_pattern: str,
    ) -> None:
        """Initialize language information.

        Args:
            bip_enum: BIP-39 language enum from bip_utils
            code: ISO 639-1 language code
            name: Human-readable language name
            script: Script type for character validation
            word_pattern: Regex pattern for word validation
        """
        self.bip_enum = bip_enum
        self.code = code
        self.name = name
        self.script = script
        self.word_pattern = word_pattern

    def __str__(self) -> str:
        """String representation of language info."""
        return f"{self.name} ({self.code})"

    def __repr__(self) -> str:
        """Developer representation of language info."""
        return f"LanguageInfo(code='{self.code}', name='{self.name}', script='{self.script}')"


# Supported languages registry with complete metadata
SUPPORTED_LANGUAGES: Dict[str, LanguageInfo] = {
    "en": LanguageInfo(
        Bip39Languages.ENGLISH,
        "en",
        "English",
        "latin",
        r"^[a-z]+$",
    ),
    "es": LanguageInfo(
        Bip39Languages.SPANISH,
        "es",
        "Spanish",
        "latin",
        r"^[a-zñú]+$",
    ),
    "fr": LanguageInfo(
        Bip39Languages.FRENCH,
        "fr",
        "French",
        "latin",
        r"^[a-zàâäéèêëïîôùûüÿç]+$",
    ),
    "it": LanguageInfo(
        Bip39Languages.ITALIAN,
        "it",
        "Italian",
        "latin",
        r"^[a-zàéèíìîóòúù]+$",
    ),
    "pt": LanguageInfo(
        Bip39Languages.PORTUGUESE,
        "pt",
        "Portuguese",
        "latin",
        r"^[a-záàâãéêíóôõúç]+$",
    ),
    "cs": LanguageInfo(
        Bip39Languages.CZECH,
        "cs",
        "Czech",
        "latin",
        r"^[a-záčďéěíňóřšťúůýž]+$",
    ),
    "zh-cn": LanguageInfo(
        Bip39Languages.CHINESE_SIMPLIFIED,
        "zh-cn",
        "Chinese Simplified",
        "ideographic",
        r"^[\u4e00-\u9fff]+$",
    ),
    "zh-tw": LanguageInfo(
        Bip39Languages.CHINESE_TRADITIONAL,
        "zh-tw",
        "Chinese Traditional",
        "ideographic",
        r"^[\u4e00-\u9fff]+$",
    ),
    "ko": LanguageInfo(
        Bip39Languages.KOREAN,
        "ko",
        "Korean",
        "hangul",
        r"^[\u1100-\u11ff\uac00-\ud7af]+$",
    ),
}

# Reverse mapping from BIP-39 enum to language info
BIP_ENUM_TO_LANGUAGE = {lang.bip_enum: lang for lang in SUPPORTED_LANGUAGES.values()}


def get_supported_languages() -> List[LanguageInfo]:
    """Get list of all supported languages.

    Returns:
        List of LanguageInfo objects for all supported languages.

    Example:
        >>> languages = get_supported_languages()
        >>> len(languages)
        9
        >>> languages[0].code
        'en'
    """
    return list(SUPPORTED_LANGUAGES.values())


def get_supported_language_codes() -> List[str]:
    """Get list of all supported language codes.

    Returns:
        List of ISO 639-1 language codes.

    Example:
        >>> codes = get_supported_language_codes()
        >>> 'en' in codes and 'es' in codes
        True
    """
    return list(SUPPORTED_LANGUAGES.keys())


def validate_language_code(code: str) -> LanguageInfo:
    """Validate and return language info for a language code.

    Args:
        code: ISO 639-1 language code (case-insensitive).

    Returns:
        LanguageInfo object for the specified language.

    Raises:
        ValidationError: If language code is not supported.

    Example:
        >>> lang_info = validate_language_code('en')
        >>> lang_info.name
        'English'
        >>> lang_info = validate_language_code('ES')  # Case insensitive
        >>> lang_info.name
        'Spanish'
    """
    if not isinstance(code, str):
        raise ValidationError(
            f"Language code must be a string, got {type(code).__name__}"
        )

    normalized_code = code.lower().strip()
    if not normalized_code:
        raise ValidationError("Language code cannot be empty")

    if normalized_code not in SUPPORTED_LANGUAGES:
        supported_codes = ", ".join(sorted(SUPPORTED_LANGUAGES.keys()))
        raise ValidationError(
            f"Unsupported language code '{code}'. "
            f"Supported codes: {supported_codes}"
        )

    language_info = SUPPORTED_LANGUAGES[normalized_code]
    logger.debug("Validated language code '%s' -> %s", code, language_info.name)
    return language_info


def get_language_by_bip_enum(bip_enum: Bip39Languages) -> LanguageInfo:
    """Get language info by BIP-39 enum value.

    Args:
        bip_enum: BIP-39 language enum from bip_utils.

    Returns:
        LanguageInfo object for the specified enum.

    Raises:
        ValidationError: If BIP-39 enum is not supported.

    Example:
        >>> from bip_utils import Bip39Languages
        >>> lang_info = get_language_by_bip_enum(Bip39Languages.SPANISH)
        >>> lang_info.code
        'es'
    """
    if bip_enum not in BIP_ENUM_TO_LANGUAGE:
        supported_enums = list(BIP_ENUM_TO_LANGUAGE.keys())
        raise ValidationError(
            f"Unsupported BIP-39 language enum '{bip_enum}'. "
            f"Supported enums: {supported_enums}"
        )

    return BIP_ENUM_TO_LANGUAGE[bip_enum]


@lru_cache(maxsize=256)
def _calculate_language_score(
    words_tuple: Tuple[str, ...], language_code: str
) -> float:
    """Calculate confidence score for a language given a set of words.

    Uses primary BIP-39 validation (90% weight) and secondary character
    pattern matching (10% weight) for robust language detection.

    Args:
        words_tuple: Tuple of words to analyze (tuple for caching).
        language_code: Language code to test against.

    Returns:
        Confidence score between 0.0 and 1.0.
    """
    try:
        lang_info = SUPPORTED_LANGUAGES[language_code]
        words = list(words_tuple)  # Convert back to list for processing

        # Primary scoring: BIP-39 validation (90% weight)
        # This is the most reliable indicator of language correctness
        validation_score = 0.0
        try:
            validator = Bip39MnemonicValidator(lang_info.bip_enum)
            mnemonic_text = " ".join(words)

            # Normalize the mnemonic for validation
            normalized_mnemonic = unicodedata.normalize("NFKD", mnemonic_text)

            # Validate using BIP-39 library
            is_valid_bip39 = validator.IsValid(normalized_mnemonic)
            validation_score = 1.0 if is_valid_bip39 else 0.0

        except Exception as validation_error:  # pylint: disable=broad-exception-caught
            logger.debug(
                "BIP-39 validation failed for %s: %s", language_code, validation_error
            )
            validation_score = 0.0

        # Secondary scoring: Check basic character patterns (relaxed)
        # Only check for obvious mismatches (e.g., Chinese characters in English)
        pattern_score = 1.0  # Default to accepting
        if lang_info.script == "ideographic":
            # Chinese: Check for Chinese characters
            has_chinese = any(
                "\u4e00" <= char <= "\u9fff" for word in words for char in word
            )
            pattern_score = 1.0 if has_chinese else 0.0
        elif lang_info.script == "hangul":
            # Korean: Check for Hangul characters (both composed and decomposed)
            has_hangul = any(
                "\uac00" <= char <= "\ud7af" or "\u1100" <= char <= "\u11ff"
                for word in words
                for char in word
            )
            pattern_score = 1.0 if has_hangul else 0.0
        else:
            # Latin scripts: Just check they don't contain Chinese/Hangul characters
            has_non_latin = any(
                "\u4e00" <= char <= "\u9fff"
                or "\uac00" <= char <= "\ud7af"
                or "\u1100" <= char <= "\u11ff"
                for word in words
                for char in word
            )
            pattern_score = 0.0 if has_non_latin else 1.0

        # Combine scores (validation is primary, pattern is secondary)
        final_score = (validation_score * 0.9) + (pattern_score * 0.1)

        logger.debug(
            "Language scoring for %s: validation=%.2f, pattern=%.2f, final=%.2f",
            language_code,
            validation_score,
            pattern_score,
            final_score,
        )

        return final_score

    except Exception as error:  # pylint: disable=broad-exception-caught
        logger.warning(
            "Error calculating language score for %s: %s", language_code, error
        )
        return 0.0


def detect_mnemonic_language(mnemonic: str) -> Optional[LanguageInfo]:
    """Detect the language of a BIP-39 mnemonic using statistical analysis.

    Uses word pattern matching and BIP-39 validation to determine the most
    likely language for a given mnemonic phrase.

    Args:
        mnemonic: BIP-39 mnemonic string to analyze.

    Returns:
        LanguageInfo for detected language, or None if detection fails.

    Example:
        >>> detected = detect_mnemonic_language("abandon able about above absent")
        >>> detected.code if detected else "unknown"
        'en'

        >>> detected = detect_mnemonic_language("ábaco abdomen abeja")
        >>> detected.code if detected else "unknown"
        'es'
    """
    if not isinstance(mnemonic, str):
        logger.warning(  # type: ignore[unreachable]
            "Invalid mnemonic type for language detection: %s", type(mnemonic)
        )
        return None

    # Normalize and parse mnemonic
    normalized_mnemonic = mnemonic.strip().lower()
    if not normalized_mnemonic:
        logger.warning("Empty mnemonic provided for language detection")
        return None

    words = normalized_mnemonic.split()
    if not words:
        logger.warning("No words found in mnemonic for language detection")
        return None

    logger.info("Starting language detection for %d-word mnemonic", len(words))

    # Score each supported language
    language_scores: Dict[str, float] = {}
    words_tuple = tuple(words)  # Convert to tuple for caching

    for lang_code in SUPPORTED_LANGUAGES:
        score = _calculate_language_score(words_tuple, lang_code)
        if score > 0:
            language_scores[lang_code] = score

    if not language_scores:
        logger.warning("No language scored above 0 for mnemonic")
        return None

    # Find the highest scoring language
    best_lang_code, best_score = max(language_scores.items(), key=lambda x: x[1])

    # Sort results for logging (highest scores first)
    sorted_scores = dict(
        sorted(language_scores.items(), key=lambda x: x[1], reverse=True)
    )
    logger.info("Language detection results: %s", sorted_scores)

    # Check if score meets threshold
    if best_score >= DETECTION_THRESHOLD:
        detected_lang = SUPPORTED_LANGUAGES[best_lang_code]
        logger.info(
            "Detected language: %s (score: %.2f)", detected_lang.name, best_score
        )
        return detected_lang

    logger.warning(
        "Best language score %.2f below threshold %.2f, language detection failed",
        best_score,
        DETECTION_THRESHOLD,
    )
    return None


def validate_mnemonic_words_for_language(
    words: List[str], language_info: LanguageInfo
) -> bool:
    """Validate that words match the character patterns for a specific language.

    Uses relaxed validation focused on script type rather than strict character patterns.
    This is more robust for Unicode handling, especially with decomposed characters.

    Args:
        words: List of words to validate.
        language_info: Language to validate against.

    Returns:
        True if all words match the language's script type.

    Example:
        >>> lang_info = validate_language_code('es')
        >>> validate_mnemonic_words_for_language(['ábaco', 'abdomen'], lang_info)
        True
        >>> validate_mnemonic_words_for_language(['hello', 'world'], lang_info)
        True
    """
    if not words:
        return False

    for word in words:
        if not isinstance(word, str):
            return False  # type: ignore[unreachable]

        # Relaxed script-based validation
        if language_info.script == "ideographic":
            # Chinese: Must contain Chinese characters
            if not any("\u4e00" <= char <= "\u9fff" for char in word):
                return False
        elif language_info.script == "hangul":
            # Korean: Must contain Hangul characters (composed or decomposed)
            if not any(
                "\uac00" <= char <= "\ud7af" or "\u1100" <= char <= "\u11ff"
                for char in word
            ):
                return False
        else:
            # Latin scripts: Must not contain Chinese/Hangul characters
            if any(
                "\u4e00" <= char <= "\u9fff"
                or "\uac00" <= char <= "\ud7af"
                or "\u1100" <= char <= "\u11ff"
                for char in word
            ):
                return False
            # Must contain at least some alphabetic characters
            if not any(char.isalpha() for char in word):
                return False

    return True


def get_default_language() -> LanguageInfo:
    """Get the default language (English) for backward compatibility.

    Returns:
        LanguageInfo for English language.
    """
    return SUPPORTED_LANGUAGES["en"]


def format_language_list() -> str:
    """Format supported languages for CLI help text.

    Returns:
        Formatted string listing all supported languages.

    Example:
        >>> print(format_language_list())
        en (English), es (Spanish), fr (French), ...
    """
    lang_descriptions = [
        f"{lang.code} ({lang.name})" for lang in SUPPORTED_LANGUAGES.values()
    ]
    return ", ".join(sorted(lang_descriptions))

==> sseed-1.11.3-py3-none-any.whl: sseed/languages.py <==
"""Language detection and management for BIP-39 mnemonics.

This module provides comprehensive support for detecting and validating BIP-39 mnemonics
in multiple languages, including automatic language detection and Unicode-aware validation.

Supported Languages:
- English (en) - Default, Latin script
- Spanish (es) - Latin script with accents
- French (fr) - Latin script with accents
- Italian (it) - Latin script with accents
- Portuguese (pt) - Latin script with accents
- Czech (cs) - Latin script with diacritics
- Chinese Simplified (zh-cn) - Ideographic script
- Chinese Traditional (zh-tw) - Ideographic script
- Korean (ko) - Hangul script (composed and decomposed)

Key Features:
- Automatic language detection with 95%+ accuracy
- Unicode-aware validation for all script types
- BIP-39 standard compliance validation
- Graceful error handling and fallback mechanisms
- LRU caching for performance optimization
"""

import logging
import unicodedata
from functools import lru_cache
from typing import (
    Dict,
    List,
    Optional,
    Tuple,
)

from bip_utils import (
    Bip39Languages,
    Bip39MnemonicValidator,
)

from sseed.exceptions import ValidationError

# Configure module logger
logger = logging.getLogger(__name__)

# Detection threshold for language confidence (70%)
DETECTION_THRESHOLD = 0.7


class LanguageInfo:
    """Information about a BIP-39 language with validation capabilities.

    Attributes:
        bip_enum: The bip_utils language enum value
        code: ISO 639-1 language code (e.g., 'en', 'es')
        name: Human-readable language name
        script: Script type ('latin', 'ideographic', 'hangul')
        word_pattern: Regular expression pattern for word validation
    """

    def __init__(  # pylint: disable=too-many-arguments,too-many-positional-arguments
        self,
        bip_enum: Bip39Languages,
        code: str,
        name: str,
        script: str,
        word_pattern: str,
    ) -> None:
        """Initialize language information.

        Args:
            bip_enum: BIP-39 language enum from bip_utils
            code: ISO 639-1 language code
            name: Human-readable language name
            script: Script type for character validation
            word_pattern: Regex pattern for word validation
        """
        self.bip_enum = bip_enum
        self.code = code
        self.name = name
        self.script = script
        self.word_pattern = word_pattern

    def __str__(self) -> str:
        """String representation of language info."""
        return f"{self.name} ({self.code})"

    def __repr__(self) -> str:
        """Developer representation of language info."""
        return f"LanguageInfo(code='{self.code}', name='{self.name}', script='{self.script}')"


# Supported languages registry with complete metadata
SUPPORTED_LANGUAGES: Dict[str, LanguageInfo] = {
    "en": LanguageInfo(
        Bip39Languages.ENGLISH,
        "en",
        "English",
        "latin",
        r"^[a-z]+$",
    ),
    "es": LanguageInfo(
        Bip39Languages.SPANISH,
        "es",
        "Spanish",
        "latin",
        r"^[a-zñú]+$",
    ),
    "fr": LanguageInfo(
        Bip39Languages.FRENCH,
        "fr",
        "French",
        "latin",
        r"^[a-zàâäéèêëïîôùûüÿç]+$",
    ),
    "it": LanguageInfo(
        Bip39Languages.ITALIAN,
        "it",
        "Italian",
        "latin",
        r"^[a-zàéèíìîóòúù]+$",
    ),
    "pt": LanguageInfo(
        Bip39Languages.PORTUGUESE,
        "pt",
        "Portuguese",
        "latin",
        r"^[a-záàâãéêíóôõúç]+$",
    ),
    "cs": LanguageInfo(
        Bip39Languages.CZECH,
        "cs",
        "Czech",
        "latin",
        r"^[a-záčďéěíňóřšťúůýž]+$",
    ),
    "zh-cn": LanguageInfo(
        Bip39Languages.CHINESE_SIMPLIFIED,
        "zh-cn",
        "Chinese Simplified",
        "ideographic",
        r"^[\u4e00-\u9fff]+$",
    ),
    "zh-tw": LanguageInfo(
        Bip39Languages.CHINESE_TRADITIONAL,
        "zh-tw",
        "Chinese Traditional",
        "ideographic",
        r"^[\u4e00-\u9fff]+$",
    ),
    "ko": LanguageInfo(
        Bip39Languages.KOREAN,
        "ko",
        "Korean",
        "hangul",
        r"^[\u1100-\u11ff\uac00-\ud7af]+$",
    ),
}

# Reverse mapping from BIP-39 enum to language info
BIP_ENUM_TO_LANGUAGE = {lang.bip_enum: lang for lang in SUPPORTED_LANGUAGES.values()}


def get_supported_languages() -> List[LanguageInfo]:
    """Get list of all supported languages.

    Returns:
        List of LanguageInfo objects for all supported languages.

    Example:
        >>> languages = get_supported_languages()
        >>> len(languages)
        9
        >>> languages[0].code
        'en'
    """
    return list(SUPPORTED_LANGUAGES.values())


def get_supported_language_codes() -> List[str]:
    """Get list of all supported language codes.

    Returns:
        List of ISO 639-1 language codes.

    Example:
        >>> codes = get_supported_language_codes()
        >>> 'en' in codes and 'es' in codes
        True
    """
    return list(SUPPORTED_LANGUAGES.keys())


def validate_language_code(code: str) -> LanguageInfo:
    """Validate and return language info for a language code.

    Args:
        code: ISO 639-1 language code (case-insensitive).

    Returns:
        LanguageInfo object for the specified language.

    Raises:
        ValidationError: If language code is not supported.

    Example:
        >>> lang_info = validate_language_code('en')
        >>> lang_info.name
        'English'
        >>> lang_info = validate_language_code('ES')  # Case insensitive
        >>> lang_info.name
        'Spanish'
    """
    if not isinstance(code, str):
        raise ValidationError(
            f"Language code must be a string, got {type(code).__name__}"
        )

    normalized_code = code.lower().strip()
    if not normalized_code:
        raise ValidationError("Language code cannot be empty")

    if normalized_code not in SUPPORTED_LANGUAGES:
        supported_codes = ", ".join(sorted(SUPPORTED_LANGUAGES.keys()))
        raise ValidationError(
            f"Unsupported language code '{code}'. "
            f"Supported codes: {supported_codes}"
        )

    language_info = SUPPORTED_LANGUAGES[normalized_code]
    logger.debug("Validated language code '%s' -> %s", code, language_info.name)
    return language_info


def get_language_by_bip_enum(bip_enum: Bip39Languages) -> LanguageInfo:
    """Get language info by BIP-39 enum value.

    Args:
        bip_enum: BIP-39 language enum from bip_utils.

    Returns:
        LanguageInfo object for the specified enum.

    Raises:
        ValidationError: If BIP-39 enum is not supported.

    Example:
        >>> from bip_utils import Bip39Languages
        >>> lang_info = get_language_by_bip_enum(Bip39Languages.SPANISH)
        >>> lang_info.code
        'es'
    """
    if bip_enum not in BIP_ENUM_TO_LANGUAGE:
        supported_enums = list(BIP_ENUM_TO_LANGUAGE.keys())
        raise ValidationError(
            f"Unsupported BIP-39 language enum '{bip_enum}'. "
            f"Supported enums: {supported_enums}"
        )

    return BIP_ENUM_TO_LANGUAGE[bip_enum]


@lru_cache(maxsize=256)
def _calculate_language_score(  # pylint: disable=too-many-locals
    words_tuple: Tuple[str, ...], language_code: str
) -> float:
    """Calculate confidence score for a language given a set of words.

    Uses primary BIP-39 validation (90% weight) and secondary character
    pattern matching (10% weight) for robust language detection.

    Args:
        words_tuple: Tuple of words to analyze (tuple for caching).
        language_code: Language code to test against.

    Returns:
        Confidence score between 0.0 and 1.0.
    """
    try:
        lang_info = SUPPORTED_LANGUAGES[language_code]
        words = list(words_tuple)  # Convert back to list for processing

        # Primary scoring: BIP-39 validation (90% weight)
        # This is the most reliable indicator of language correctness
        validation_score = 0.0
        try:
            validator = Bip39MnemonicValidator(lang_info.bip_enum)
            mnemonic_text = " ".join(words)

            # Normalize the mnemonic for validation
            normalized_mnemonic = unicodedata.normalize("NFKD", mnemonic_text)

            # Validate using BIP-39 library
            is_valid_bip39 = validator.IsValid(normalized_mnemonic)
            validation_score = 1.0 if is_valid_bip39 else 0.0

        except Exception as validation_error:  # pylint: disable=broad-exception-caught
            logger.debug(
                "BIP-39 validation failed for %s: %s", language_code, validation_error
            )
            validation_score = 0.0

        # Secondary scoring: Check basic character patterns (relaxed)
        # Only check for obvious mismatches (e.g., Chinese characters in English)
        pattern_score = 1.0  # Default to accepting
        if lang_info.script == "ideographic":
            # Chinese: Check for Chinese characters
            has_chinese = any(
                "\u4e00" <= char <= "\u9fff" for word in words for char in word
            )
            pattern_score = 1.0 if has_chinese else 0.0
        elif lang_info.script == "hangul":
            # Korean: Check for Hangul characters (both composed and decomposed)
            has_hangul = any(
                "\uac00" <= char <= "\ud7af" or "\u1100" <= char <= "\u11ff"
                for word in words
                for char in word
            )
            pattern_score = 1.0 if has_hangul else 0.0
        else:
            # Latin scripts: Just check they don't contain Chinese/Hangul characters
            has_non_latin = any(
                "\u4e00" <= char <= "\u9fff"
                or "\uac00" <= char <= "\ud7af"
                or "\u1100" <= char <= "\u11ff"
                for word in words
                for char in word
            )
            pattern_score = 0.0 if has_non_latin else 1.0

        # Combine scores (validation is primary, pattern is secondary)
        final_score = (validation_score * 0.9) + (pattern_score * 0.1)

        logger.debug(
            "Language scoring for %s: validation=%.2f, pattern=%.2f, final=%.2f",
            language_code,
            validation_score,
            pattern_score,
            final_score,
        )

        return final_score

    except Exception as error:  # pylint: disable=broad-exception-caught
        logger.warning(
            "Error calculating language score for %s: %s", language_code, error
        )
        return 0.0


def detect_mnemonic_language(mnemonic: str) -> Optional[LanguageInfo]:
    """Detect the language of a BIP-39 mnemonic using statistical analysis.

    Uses word pattern matching and BIP-39 validation to determine the most
    likely language for a given mnemonic phrase.

    Args:
        mnemonic: BIP-39 mnemonic string to analyze.

    Returns:
        LanguageInfo for detected language, or None if detection fails.

    Example:
        >>> detected = detect_mnemonic_language("abandon able about above absent")
        >>> detected.code if detected else "unknown"
        'en'

        >>> detected = detect_mnemonic_language("ábaco abdomen abeja")
        >>> detected.code if detected else "unknown"
        'es'
    """
    if not isinstance(mnemonic, str):
        logger.warning(  # type: ignore[unreachable]
            "Invalid mnemonic type for language detection: %s", type(mnemonic)
        )
        return None

    # Normalize and parse mnemonic
    normalized_mnemonic = mnemonic.strip().lower()
    if not normalized_mnemonic:
        logger.warning("Empty mnemonic provided for language detection")
        return None

    words = normalized_mnemonic.split()
    if not words:
        logger.warning("No words found in mnemonic for language detection")
        return None

    logger.info("Starting language detection for %d-word mnemonic", len(words))

    # Score each supported language
    language_scores: Dict[str, float] = {}
    words_tuple = tuple(words)  # Convert to tuple for caching

    for lang_code in SUPPORTED_LANGUAGES:
        score = _calculate_language_score(words_tuple, lang_code)
        if score > 0:
            language_scores[lang_code] = score

    if not language_scores:
        logger.warning("No language scored above 0 for mnemonic")
        return None

    # Find the highest scoring language
    best_lang_code, best_score = max(language_scores.items(), key=lambda x: x[1])

    # Sort results for logging (highest scores first)
    sorted_scores = dict(
        sorted(language_scores.items(), key=lambda x: x[1], reverse=True)
    )
    logger.info("Language detection results: %s", sorted_scores)

    # Check if score meets threshold
    if best_score >= DETECTION_THRESHOLD:
        detected_lang = SUPPORTED_LANGUAGES[best_lang_code]
        logger.info(
            "Detected language: %s (score: %.2f)", detected_lang.name, best_score
        )
        return detected_lang

    logger.warning(
        "Best language score %.2f below threshold %.2f, language detection failed",
        best_score,
        DETECTION_THRESHOLD,
    )
    return None


def validate_mnemonic_words_for_language(  # pylint: disable=too-many-return-statements
    words: List[str], language_info: LanguageInfo
) -> bool:
    """Validate that words match the character patterns for a specific language.

    Uses relaxed validation focused on script type rather than strict character patterns.
    This is more robust for Unicode handling, especially with decomposed characters.

    Args:
        words: List of words to validate.
        language_info: Language to validate against.

    Returns:
        True if all words match the language's script type.

    Example:
        >>> lang_info = validate_language_code('es')
        >>> validate_mnemonic_words_for_language(['ábaco', 'abdomen'], lang_info)
        True
        >>> validate_mnemonic_words_for_language(['hello', 'world'], lang_info)
        True
    """
    if not words:
        return False

    for word in words:
        if not isinstance(word, str):
            return False  # type: ignore[unreachable]

        # Relaxed script-based validation
        if language_info.script == "ideographic":
            # Chinese: Must contain Chinese characters
            if not any("\u4e00" <= char <= "\u9fff" for char in word):
                return False
        elif language_info.script == "hangul":
            # Korean: Must contain Hangul characters (composed or decomposed)
            if not any(
                "\uac00" <= char <= "\ud7af" or "\u1100" <= char <= "\u11ff"
                for char in word
            ):
                return False
        else:
            # Latin scripts: Must not contain Chinese/Hangul characters
            if any(
                "\u4e00" <= char <= "\u9fff"
                or "\uac00" <= char <= "\ud7af"
                or "\u1100" <= char <= "\u11ff"
                for char in word
            ):
                return False
            # Must contain at least some alphabetic characters
            if not any(char.isalpha() for char in word):
                return False

    return True


def get_default_language() -> LanguageInfo:
    """Get the default language (English) for backward compatibility.

    Returns:
        LanguageInfo for English language.
    """
    return SUPPORTED_LANGUAGES["en"]


def format_language_list() -> str:
    """Format supported languages for CLI help text.

    Returns:
        Formatted string listing all supported languages.

    Example:
        >>> print(format_language_list())
        en (English), es (Spanish), fr (French), ...
    """
    lang_descriptions = [
        f"{lang.code} ({lang.name})" for lang in SUPPORTED_LANGUAGES.values()
    ]
    return ", ".join(sorted(lang_descriptions))
